#! /usr/bin/env python
# -*- coding: utf-8 -*-

# run3dcase.py runs simpleFoam with k-epsilon closure for 3D case defined in the template0 directory
# run3dcase.py uses templates included in the case, and changes the boundary conditions and creates a mesh according 
# to specified arguments with SHM
# example call to function:
# run3dcase.py template0, inletX, z0, us, h, caseType 
# run3dcase.py 3DBumpTemplate 1400 0.03 0.15 60 regular

# where
# template 	- the case with the appropriate dictionaries, templates and boundary conditions   
# inletX	- location for sampling inlet Ux profile of simulation [m]	   
# z0   		- roughness length (same for all of the ground) [m]
# us  		- friction velocity - for inlet conditions [m/s]
# h  		- height of hill - assumed in location (0,0,h) [m]
# caseType	- if "mapFields" then initial guess will be taken from a less refined solution

import math, os, multiprocessing
from os import path
from PyFoam.RunDictionary.SolutionDirectory 	import SolutionDirectory
from PyFoam.Basics.TemplateFile 		import TemplateFile
from PyFoam.RunDictionary.ParsedParameterFile 	import ParsedParameterFile
from PyFoam.Basics.DataStructures 		import Vector
from PyFoam.Execution.BasicRunner 		import BasicRunner
from PyFoam.Applications.PlotRunner 		import PlotRunner
from PyFoam.Applications.Runner 		import Runner
from PyFoam.Applications.Decomposer 		import Decomposer
from PyFoam.Applications.CaseReport		import CaseReport
from PyFoam.Execution.ParallelExecution		import LAMMachine
from PyFoam.Applications.PotentialRunner	import PotentialRunner
from numpy import *
from pylab import *
import matplotlib.pyplot as plt
import os,glob,subprocess
from matplotlib.backends.backend_pdf import PdfPages
import pdb
b = pdb.set_trace

# reading arguments
n = len(sys.argv)

if n<5:
    print "need 3DBumpTemplate 1400 0.03 0.15 60 "
template0 = sys.argv[1]
inletX = float(sys.argv[2])
z0 = float(sys.argv[3])
us = float(sys.argv[4])
h = float(sys.argv[5])

def run3dcase(template0, inletX, z0, us, h):
	
	# case definitions (Martinez 2011)
	ks = 19.58 * z0 # [m] Martinez 2011
	k = 0.4
	Cmu = 0.03 	# Castro 96
	Href = 10000 # any number higher then the top boundary height

	procnr = multiprocessing.cpu_count()
	caseStr = template0 + "_h_" + str(h)  	
  	target = template0 + "/runs/" + caseStr
	
	#--------------------------------------------------------------------------------------
	# cloaning case
	#--------------------------------------------------------------------------------------
	orig = SolutionDirectory(template0,
			  archive=None,
			  paraviewLink=False)
	work = orig.cloneCase(target)
				  
	#--------------------------------------------------------------------------------------
	# changing inlet profile - - - - according to Martinez 2010
	#--------------------------------------------------------------------------------------
	# change inlet profile
	Uref = Utop = us/k*math.log(Href/z0)
	# calculating turbulentKE
	TKE = us*us/math.sqrt(Cmu)
	# 1: changing ABLConditions
	bmName = path.join(work.initialDir(),"include/ABLConditions")
	template = TemplateFile(bmName+".template")
	template.writeToFile(bmName,{'us':us,'Uref':Uref,'Href':Href,'z0':z0})
	# 2: changing initialConditions
	bmName = path.join(work.initialDir(),"include/initialConditions")
	template = TemplateFile(bmName+".template")
	template.writeToFile(bmName,{'TKE':TKE})

	# 3: changing initial and boundary conditions for new z0
	# changing ks in nut, inside nutRoughWallFunction
	nutFile = ParsedParameterFile(path.join(work.initialDir(),"nut"))
	nutFile["boundaryField"]["ground"]["Ks"].setUniform(ks)
	nutFile.writeFile()
	
	#--------------------------------------------------------------------------------------
	# changing sample file
	#--------------------------------------------------------------------------------------
	bmName = path.join(work.systemDir(),"sampleDict")
	template = TemplateFile(bmName+".template")
	template.writeToFile(bmName,{'hillTopY':h,'sampleHeightAbovePlain':50,'sampleHeightAboveHill':h+50,'inletX':inletX}) 
	
	#--------------------------------------------------------------------------------------
	# creating blockMesh
	#--------------------------------------------------------------------------------------
	blockRun = BasicRunner(argv=["blockMesh",'-case',work.name],silent=True,server=False,logname="blockMesh")
	print "Running blockMesh"
	blockRun.start()
	if not blockRun.runOK(): error("there was an error with blockMesh")

	#--------------------------------------------------------------------------------------
	# running SHM
	#--------------------------------------------------------------------------------------
	SHMrun = BasicRunner(argv=["snappyHexMesh",'-overwrite','-case',work.name],server=False,logname="SHM")
	print "Running SHM"
	SHMrun.start()

	if procnr>1:
		#--------------------------------------------------------------------------------------
		# decomposing
		#--------------------------------------------------------------------------------------
		if 1:
			decomposeParName = path.join(work.systemDir(),"decomposeParDict")
			template = TemplateFile(decomposeParName+".template")
	
			if procnr==2:
				n1,n2,n3=2,1,1
			elif procnr==3:
				n1,n2,n3=3,1,1
			elif procnr==4:
				n1,n2,n3=2,1,2
			elif procnr==6:
				n1,n2,n3=3,2,1
			elif procnr==8:
				n1,n2,n3=2,2,2
			elif procnr%4==0:
				n1,n2,n3=procnr/4,2,2
			elif procnr%2==0:
				n1,n2,n3=procnr/2,2,1
			else:
				n1,n2,n3=procnr,1,1
			template.writeToFile(decomposeParName,{'n':procnr,'n1':n1,'n2':n2,'n3':n3}) 


		# removing U.template from 0/ directory
		subprocess.call("rm " + bmName + ".template ",shell=True)
		arg = " -case " + work.name
	 	decomposeRun = BasicRunner(argv=["decomposePar -force" + arg],silent=True,server=False,logname="decompose")
		decomposeRun.start()


		#--------------------------------------------------------------------------------------
		# running
		#--------------------------------------------------------------------------------------
		machine = LAMMachine(nr=procnr)
		# run case
		PlotRunner(args=["--proc=%d"%procnr,"--progress","--no-continuity","--hardcopy", "--non-persist", "simpleFoam","-case",work.name])

		#--------------------------------------------------------------------------------------
		# reconstruct
		#-------------------------------------------------------------------------
		reconstructRun = BasicRunner(argv=["reconstructPar -latestTime" + arg],server=False,logname="reconstructLog")
		reconstructRun.start()

	else:
		
		#--------------------------------------------------------------------------------------
		# running
		#--------------------------------------------------------------------------------------
		PlotRunner(args=["--progress","simpleFoam","-case",work.name])

	#--------------------------------------------------------------------------------------
	# sample results
	#--------------------------------------------------------------------------------------
	#dirName = str(glob.glob(target))
	# sampling
	arg = " -case " + str(target)
	sampleRun = BasicRunner(argv=["sample -latestTime" + arg],server=False,logname="sampleLog")
	sampleRun.start()

 	#finding the most converged run.
  	setName =  glob.glob(target + '/sets/*')

  	lastRun = range(len(setName))
	for num in range(len(setName)):
		lastRun[num] = int(setName[num][setName[num].rfind("/")+1:])
  	m = max(lastRun)
  	p = lastRun.index(m)
	data_y = genfromtxt(setName[p] + '/line_y_U.xy',delimiter=' ')
  	y, Ux_y, Uy_y  = data_y[:,0], data_y[:,1], data_y[:,2] 
	y = y-h # normalizing data to height of hill-top above ground
	
	#--------------------------------------------------------------------------------------
	# plot results
	#--------------------------------------------------------------------------------------
	import plot3DBump
	plot3DBump.main(h,[target])
	
	return y,Ux_y,Uy_y

if __name__ == '__main__':
 run3dcase(template0, inletX, z0, us, h)
